package com.unissoft.alarm.controller;


import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.alarm.model.AlarmRecord;
import com.unissoft.alarm.service.AlarmRecordService;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * 报警记录表 前端控制器
 * </p>
 *
 * @author root
 * @since 2022-06-27
 */
@Api(tags = "报警记录")
@RestController
@RequestMapping("/alarm-record")
public class AlarmRecordController {

	@Resource
    private AlarmRecordService alarmRecordService;

    @PostMapping("/getPageList")
    @ApiOperation(value = "报警记录", notes = "分页列表")
    public ResultView getPageList(@RequestBody PageParam pageParam) {
        try {
            IPage<AlarmRecord> pageList = alarmRecordService.getPageList(pageParam);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }
    
    
    /**
     * 处理
     */
    @PostMapping("/update")
    @ApiOperation(value = "报警记录", notes = "修改")
    public ResultView update(@RequestBody AlarmRecord alarmRecord) {
        try {
            alarmRecordService.updateAlarmRecordById(alarmRecord);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }
    
    /**
     * 查看
     */
    @ApiOperation(value = "报警记录", notes = "根据id查询")
    @GetMapping("/getByIdList/{id}")
    public ResultView getByIdList(@PathVariable Long id) {
    	AlarmRecord alarmRecord = alarmRecordService.queryById(id);
        return ResultView.success(alarmRecord);
    }
    
    /**
     *新增
     */
    /*@PostMapping("/save")
    @ApiOperation(value = "报警记录", notes = "新增")
    public ResultView save(@RequestBody AlarmRecord alarmRecord) {
        try {
        	alarmRecordService.saveAlarmRecord(alarmRecord);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }*/
    
    /**
     * 首页 待处理报警
     * @param orgCode
     * @return
     */
    @GetMapping("/getTodoList/{orgCode}")
    @ApiOperation(value = "待处理报警", notes = "待处理报警")
    public ResultView getTodoList(@PathVariable @NotNull String orgCode) {
        try {
            return ResultView.success(alarmRecordService.getTodoList(orgCode));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }
    
    
}
