package com.unissoft.alarm.service.impl;

import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.alarm.model.AlarmRecord;
import com.unissoft.alarm.mapper.AlarmRecordMapper;
import com.unissoft.alarm.service.AlarmRecordService;
import com.unissoft.basic.mapper.AlarmParamMapper;
import com.unissoft.basic.model.AlarmParam;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import com.unissoft.deviceManage.model.DeviceDataPoint;
import com.unissoft.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.redisson.client.RedisNodeNotFoundException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 报警记录表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-27
 */
@Slf4j
@Service
public class AlarmRecordServiceImpl extends ServiceImpl<AlarmRecordMapper, AlarmRecord> implements AlarmRecordService {

    @Resource
    private AlarmRecordMapper alarmRecordMapper;
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private AlarmParamMapper alarmParamMapper;
    @Value("${spring.redis.six_hours}")
    private long SIX_HOURS;

    @Override
    public IPage<AlarmRecord> getPageList(PageParam pageParam) {
        // TODO Auto-generated method stub

        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        // 分页对象
        IPage<AlarmRecord> page = new Page<>(pageIndex, pageSize);
        // 查询条件
        String condition = pageParam.getCondition();
        QueryWrapper<AlarmRecord> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(condition)) {
            com.alibaba.fastjson.JSONObject jsStr = JSON.parseObject(condition);
            String orgCode = jsStr.getString("orgCode");//组织code
            String proCode = jsStr.getString("proCode");//产品名称
            String eqCode = jsStr.getString("eqCode");//设备名称
            String labelCode = jsStr.getString("labelCode");//设备标签
            String alarmTypeCode = jsStr.getString("alarmTypeCode");//报警类型
            String handleState = jsStr.getString("handleState");//处理状态
            if (!StringUtils.isEmpty(orgCode)) {
                ew.eq("ar.org_code", orgCode);
            }
            if (!StringUtils.isEmpty(proCode)) {
                ew.eq("ar.pro_code", proCode);
            }
            if (!StringUtils.isEmpty(eqCode)) {
                ew.eq("ar.eq_code", eqCode);
            }
            if (!StringUtils.isEmpty(labelCode)) {
                ew.eq("ar.label_code", labelCode);
            }
            if (!StringUtils.isEmpty(alarmTypeCode)) {
                ew.eq("ar.alarm_type_code", alarmTypeCode);
            }
            if (!StringUtils.isEmpty(handleState)) {
                ew.eq("ar.handle_state", handleState);
            }
        }
        ew.orderByAsc("handle_state");// 未处理优先
        return alarmRecordMapper.selectPageList(page, ew);

    }

    @Override
    public List<AlarmRecord> getTodoList(String orgCode) {
        QueryWrapper<AlarmRecord> ew = new QueryWrapper<>();
        // 待处理
        ew.eq("handle_state", "0");
        // 传入参数非空
        ew.eq("org_code", orgCode);
        return alarmRecordMapper.selectList(ew);
    }

    @Override
    public AlarmRecord queryById(Long id) {
        // TODO Auto-generated method stub
        return alarmRecordMapper.queryById(id);
    }

    @Override
    public void saveAlarmRecord(String sn, List<DeviceDataPoint> deviceDataPoints) {
        // 设备SN编号获取机构编号orgCode
        if (redisUtil.hHasKey(MyConstant.REDIS_X1_ORG, sn)) {
            String orgCode = redisUtil.hget(MyConstant.REDIS_X1_ORG, sn).toString();
            // 查询预警配置
            if (StringUtils.isEmpty(orgCode))
                throw new RedisNodeNotFoundException("redis is not found key");
            deviceDataPoints.forEach(devData -> {
                String device = devData.getDevice();
                // 查看是否有记录
                if (!redisUtil.hHasKey(MyConstant.REDIS_ALARM_RECORD, device)) {
                    List<AlarmParam> setAlarmList = alarmParamMapper.getCheckForEqCode(orgCode, String.format("%s%s%s", "%", device, "%"));
                    if (setAlarmList.size() > 0) {
                        // 去重
                        List<AlarmParam> result = setAlarmList.stream().collect(
                                Collectors.collectingAndThen(
                                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AlarmParam::getId))), ArrayList::new
                                )
                        );
                        // 预警处理,比对数据 deviceDataPoints-->result
                        result.forEach(setData -> {
                            // 温湿度高限
                            Double alarmValueMax = setData.getAlarmValueMax().doubleValue();
                            // 温湿度底限
                            Double alarmValueMin = setData.getAlarmValueMin().doubleValue();
                            // 温度值
                            Double tempValue = devData.getTempValue();
                            // 湿度值
                            Double wetValue = devData.getWetValue();
                            if (tempValue >= alarmValueMax || tempValue <= alarmValueMin
                                    || wetValue >= alarmValueMax || wetValue <= alarmValueMin) {
                                // 查看是否有记录
                                log.info("温湿度预警处理消息数据: {} -> {}", sn, JSON.toJSONString(deviceDataPoints));
                                AlarmRecord record = new AlarmRecord();
                                record.setId(YitIdHelper.nextId()); // 主键
                                record.setOrgCode(orgCode); // 机构编码
                                record.setProCode(sn); // 产品编码
                                record.setLabelCode(setData.getLabelCode()); // 设备标签
                                record.setEqCode(device); // 设备编码
                                record.setAlarmValueMax(BigDecimal.valueOf(alarmValueMax)); // 预警高限
                                record.setAlarmValueMin(BigDecimal.valueOf(alarmValueMin)); // 预警底限
                                record.setAlarmTime(new Date()); // 报警系统写入时间
                                // 温度报警:110001
                                String alarmTypeCode = setData.getAlarmTypeCode();
                                if ("110001".equals(alarmTypeCode)) {
                                    record.setAlarmTypeCode(alarmTypeCode); // 报警类型
                                    // 温度对比,保存报警
                                    if (tempValue >= alarmValueMax) { // 检测值大于等于预警值-->高限报警
                                        record.setAlarmRemark("温度高于阀值");
                                        alarmRecordMapper.insert(record);
                                    }
                                    if (tempValue <= alarmValueMin) { // 检测值小于等于预警值-->底限报警
                                        record.setAlarmRemark("温度底于阀值");
                                        alarmRecordMapper.insert(record);
                                    }
                                }
                                // 湿度报警:110002
                                if ("110002".equals(alarmTypeCode)) {
                                    record.setAlarmTypeCode(alarmTypeCode); // 报警类型
                                    if (wetValue >= alarmValueMax) { // 检测值大于等于预警值-->高限报警
                                        record.setAlarmRemark("湿度高于阀值");
                                        alarmRecordMapper.insert(record);
                                    }
                                    if (wetValue <= alarmValueMin) { // 检测值小于等于预警值-->底限报警
                                        record.setAlarmRemark("湿度底于阀值");
                                        alarmRecordMapper.insert(record);
                                    }
                                }
                                // 设置缓存,在处理方法中,清除缓存
                                this.synRedisAlarmRecord("hset", device, record);
                            }
                        });
                    }
                }
            });
        }

    }

    @Override
    public void updateAlarmRecordById(AlarmRecord alarmRecord) {
        alarmRecord.setHandleState("1"); //已处理
        alarmRecord.setHandleTime(new Date());
        alarmRecordMapper.updateById(alarmRecord);
        // 清除缓存
        String eqCode = alarmRecord.getEqCode();
        if (!StringUtils.isEmpty(eqCode))
            if (redisUtil.hHasKey(MyConstant.REDIS_ALARM_RECORD, eqCode))
                this.synRedisAlarmRecord("hdel", eqCode, null);
    }

    @Override
    public List<Map> alarmRecordVis(String orgCode, String labelCode, Integer pageSize) {
        return alarmRecordMapper.alarmRecordVis(orgCode, labelCode, pageSize);
    }

    // redis 添加删除同步锁(针对温湿度报警),解决redis has 值删除和新增并发问题
    private void synRedisAlarmRecord(String lockType, String eqCode, AlarmRecord record) {
        synchronized (MyConstant.REDIS_ALARM_RECORD) {
            switch (lockType) {
                case "hdel":
                    redisUtil.hdel(MyConstant.REDIS_ALARM_RECORD, eqCode);
                    break;
                case "hset":
                    // 设置缓存,在处理方法中,清除缓存
                    redisUtil.hset(MyConstant.REDIS_ALARM_RECORD, eqCode, record, SIX_HOURS);
                    break;
                default:
                    throw new IllegalArgumentException("redis execute hash key to item error in REDIS_ALARM_RECORD");
            }
        }
    }

}
