package com.unissoft.basic.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.basic.model.TimingTask;
import com.unissoft.basic.model.TimingTaskList;
import com.unissoft.basic.model.TimingTaskReq;
import com.unissoft.basic.service.TimingTaskService;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.common.SystemLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 定时任务设置表 前端控制器
 * </p>
 *
 * @author root
 * @since 2022-06-24
 */
@Api(tags = "参数设置")
@RestController
@RequestMapping("/timing-task")
public class TimingTaskController {
    @Resource
    private TimingTaskService timingTaskService;

    @PostMapping("/getPageList")
    @ApiOperation(value = "定时任务设置", notes = "分页列表,orgCode 非空")
    public ResultView getPageList(@RequestBody PageParam pageParam) {
        try {
            IPage<TimingTask> pageList = timingTaskService.getPageList(pageParam);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @PostMapping("/save")
    @ApiOperation(value = "定时任务设置", notes = "新增")
    @SystemLog(operModul = "参数设置", operType = "定时任务设置", operDesc = "新增参数")
    public ResultView saveList(@RequestBody TimingTaskList timingTaskList) {
        try {
            timingTaskService.saveList(timingTaskList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

    @PostMapping("/update")
    @ApiOperation(value = "定时任务设置", notes = "修改")
    @SystemLog(operModul = "参数设置", operType = "定时任务设置", operDesc = "修改参数")
    public ResultView saveOrUpdate(@RequestBody TimingTaskList timingTaskList) {
        try {
            timingTaskService.updateList(timingTaskList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

    @ApiOperation(value = "定时任务设置", notes = "根据id查询")
    @GetMapping("/getByIdList/{id}")
    public ResultView getByIdList(@PathVariable Long id) {
        TimingTaskList timingTaskList = timingTaskService.getByIdList(id);
        return ResultView.success(timingTaskList);
    }

    @GetMapping("/deleteById/{id}")
    @ApiOperation(value = "定时任务设置", notes = "根据id删除")
    @SystemLog(operModul = "参数设置", operType = "定时任务设置", operDesc = "删除参数")
    public ResultView deleteById(@PathVariable Long id) {
        if (timingTaskService.deleteById(id)) {
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @GetMapping("/getByTask")
    @ApiOperation(value = "设备管理", notes = "设备-定时任务")
    public ResultView getByTask(@RequestParam @NotNull String orgCode, @RequestParam @NotNull String eqCode) {
        return ResultView.success(timingTaskService.getByTask(orgCode, eqCode));
    }


    /*
     * 平台调用设置定时任务接口 post
     * saveTask:新增
     * updateTask:修改
     * deleteTask/{id} restFull
     */

    @PostMapping("/saveTask")
    @SystemLog(operModul = "平台调用设置", operType = "定时任务设置", operDesc = "新增参数")
    public ResultView saveTask(@RequestBody TimingTaskReq timingTaskReq) {
        Long id;
        try {
            id = timingTaskService.saveTask(timingTaskReq);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success(id);
    }

    @PostMapping("/updateTask")
    @SystemLog(operModul = "平台调用设置", operType = "定时任务设置", operDesc = "新增参数")
    public ResultView updateTask(@RequestBody TimingTaskReq timingTaskReq) {
        try {
            timingTaskService.updateTask(timingTaskReq);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

}
