package com.unissoft.basic.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.basic.model.AlarmParam;
import com.unissoft.basic.mapper.AlarmParamMapper;
import com.unissoft.basic.model.AlarmParamList;
import com.unissoft.basic.service.AlarmParamService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.PageParam;
import com.unissoft.deviceManage.mapper.DeviceInfoMapper;
import com.unissoft.deviceManage.model.DeviceInfo;
import org.bouncycastle.math.ec.custom.sec.SecT113Field;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警参数设置表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-21
 */
@Service
public class AlarmParamServiceImpl extends ServiceImpl<AlarmParamMapper, AlarmParam> implements AlarmParamService {
    @Resource
    private AlarmParamMapper alarmParamMapper;
    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Override
    public IPage<AlarmParam> getPageList(PageParam pageParam) {
        // 创建分页条件对象
        IPage<AlarmParam> page = new Page<>(pageParam.getPageIndex(), pageParam.getPageSize());
        // 查询对象
        QueryWrapper<AlarmParam> qw = new QueryWrapper<>();
        // 解析查询条件
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            AlarmParam alarmParam = JSON.parseObject(condition, AlarmParam.class);
            qw.eq("org_code", alarmParam.getOrgCode());
        }
        return alarmParamMapper.selectPage(page, qw);
    }

    @Override
    public long saveList(AlarmParamList alarmParamList) {
        // 字符串拼接
        joiningString(alarmParamList);
        // 保存
        long id = YitIdHelper.nextId();
        alarmParamList.setId(id);
        alarmParamMapper.insert(alarmParamList);
        return id;
    }

    @Override
    public void updateList(AlarmParamList alarmParamList) {
        /* 字符串拼接 */
        joiningString(alarmParamList);
        // 保存
        alarmParamMapper.updateById(alarmParamList);
    }

    @Override
    @SuppressWarnings("all")
    public AlarmParamList getByIdList(Long id) {
        // 返回子类对象
        AlarmParamList alarmParamList = new AlarmParamList();
        // 查询主信息
        AlarmParam alarmParam = alarmParamMapper.selectById(id);
        if (Objects.nonNull(alarmParam)) {
            // 查询设备信息
            List<DeviceInfo> list = deviceInfoMapper.selectInEqCodeList(
                    alarmParam.getEqCode().split(","), alarmParam.getOrgCode(), alarmParam.getLabelCode()
            );
            // 父类转子类
            BeanUtils.copyProperties(alarmParam, alarmParamList);
            // 添加设备信息
            if (!CollectionUtils.isEmpty(list))
                alarmParamList.setDevInfoList(list);
        }
        return alarmParamList;
    }

    @Override
    public List<AlarmParam> alarmParamVis(String orgCode, String labelCode) {
        return alarmParamMapper.alarmParamVis(orgCode, labelCode);
    }

    private synchronized void joiningString(AlarmParamList alarmParamList) {
        // 获取设备信息并拼接eqCode
        List<DeviceInfo> devInfoList = alarmParamList.getDevInfoList();
        if (!CollectionUtils.isEmpty(devInfoList)) {
            alarmParamList.setEqCode(
                    devInfoList.stream().map(DeviceInfo::getEqCode).collect(Collectors.joining(","))
            );
            // 设备标签编码,多设备必须为同一设备标签
            alarmParamList.setLabelCode(devInfoList.get(0).getLabelCode());
            // 防止前端cookie丢失orgCode
            if (ObjectUtils.isEmpty(alarmParamList.getOrgCode()))
                alarmParamList.setOrgCode(devInfoList.get(0).getOrgCode());
        }
    }

}
