package com.unissoft.basic.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.basic.mapper.BasicEnumMapper;
import com.unissoft.basic.model.BasicEnum;
import com.unissoft.basic.service.BasicEnumService;
import com.unissoft.common.PageParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@Service
public class BasicEnumServiceImpl extends ServiceImpl<BasicEnumMapper, BasicEnum> implements BasicEnumService {

	private BasicEnumMapper enumMapper;
	@Autowired
	public void setBasicEnumService(BasicEnumMapper enumMapper) {
        this.enumMapper = enumMapper;
    }
	
	@Override
	public IPage<BasicEnum> getPage(PageParam pageParam) {
		// TODO Auto-generated method stub
		Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        // 分页对象
        IPage<BasicEnum> page = new Page<>(pageIndex, pageSize);
        // 查询条件
        String condition = pageParam.getCondition();
        QueryWrapper<BasicEnum> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(condition)) {
        	com.alibaba.fastjson.JSONObject jsStr = JSON.parseObject(condition);
    		String enumName = jsStr.getString("enumName");//名称
    		if(!StringUtils.isEmpty(enumName)){
    			ew.like("enum_name", enumName);
    		}
        }
        return enumMapper.selectPage(page, ew);
	}

	@Override
	public List<BasicEnum> queryByTree(PageParam pageParam) {
		// TODO Auto-generated method stub
		QueryWrapper<BasicEnum> ew = new QueryWrapper<>();
		String condition = pageParam.getCondition();
		/*if (!StringUtils.isEmpty(condition)) {
	        
        	JSONObject jsStr = JSON.parseObject(condition);
    		String enumName = jsStr.getString("enumName");//名称
    		if(!StringUtils.isEmpty(enumName)){
    			ew.like("enum_name", enumName);
    		}
        }*/
		ew.orderByAsc("parent_id");
		List<BasicEnum> list = enumMapper.selectList(ew); //查询到满足条件所有的数据
		List<BasicEnum> firstlist = this.getChildren(list, 0L,condition);
		this.fillTree(list, firstlist);
		return firstlist;
	}
	
	private List<BasicEnum> getChildren(List<BasicEnum> list, Long parentId, String condition) {
        String  enumName = null;
        String  enumCode = null;
		if (!StringUtils.isEmpty(condition)) {
        	JSONObject jsStr = JSON.parseObject(condition);
    		 enumName = jsStr.getString("enumName");//名称
    		 enumCode = jsStr.getString("enumCode");//code
        }
		List<BasicEnum> children = new ArrayList<BasicEnum>();
        for(BasicEnum be: list){
            //遍历出父id等于参数的id，add进子节点集合 
        	if(null != enumName && !"".equals(enumName)){
        		if(be.getEnumName().contains(enumName)){
                	children.add(be);
            	}
        	}else if(null != enumCode && !"".equals(enumCode)){
        		if(be.getEnumCode().equals(enumCode)){
        			children.add(be);
        		}
        		
        	}else{
        		if (be.getParentId().equals(parentId)) {
            		children.add(be);
            	}
        	}
        }
        return children;
	}
	
	public void fillTree(List<BasicEnum> allList, List<BasicEnum> childList) {
        if(childList.size()>0){
            for(BasicEnum be : childList) {
                List<BasicEnum> childList1 = this.getChildren(allList, be.getId(),null);
                fillTree(allList, childList1);
                be.setChildren(childList1);
            }
        }
    }

}
