package com.unissoft.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.basic.model.RemoteParams;
import com.unissoft.basic.mapper.RemoteParamsMapper;
import com.unissoft.basic.service.RemoteParamsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.PageParam;
import com.unissoft.utils.MapConvertUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-07-12
 */
@Slf4j
@Service
public class RemoteParamsServiceImpl extends ServiceImpl<RemoteParamsMapper, RemoteParams> implements RemoteParamsService {
    @Resource
    private RemoteParamsMapper paramsMapper;
    @Resource
    private MapConvertUtils mapConvertUtils;

    @Override
    public IPage<RemoteParams> getPageList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        // 查询条件
        QueryWrapper<RemoteParams> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("rp.id");
        // 分页对象
        IPage<RemoteParams> page = new Page<>(pageIndex, pageSize);
        return paramsMapper.selectPageList(page, queryWrapper);
    }

    @Override
    public void saveAndUpdate(RemoteParams remoteParams) {
        if (Objects.isNull(remoteParams.getId())) {
            remoteParams.setId(YitIdHelper.nextId());
            remoteParams.setBuildTime(new Date());
            paramsMapper.insert(remoteParams);
        } else {
            paramsMapper.updateById(remoteParams);
        }
    }

    @Override
    public String getByTopics() {
        return mapConvertUtils.joiningList(paramsMapper.selectTopics());
    }

}
