package com.unissoft.basic.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.basic.model.SiteType;
import com.unissoft.basic.mapper.SiteTypeMapper;
import com.unissoft.basic.service.SiteTypeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.PageParam;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;

/**
 * <p>
 * 站点类型表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-23
 */
@Service
public class SiteTypeServiceImpl extends ServiceImpl<SiteTypeMapper, SiteType> implements SiteTypeService {
    @Resource
    private SiteTypeMapper siteTypeMapper;

    @Override
    public IPage<SiteType> getPageList(PageParam pageParam) {
        // 创建分页条件对象
        IPage<SiteType> page = new Page<>(pageParam.getPageIndex(), pageParam.getPageSize());
        // 查询对象
        QueryWrapper<SiteType> qw = new QueryWrapper<>();
        // 解析查询条件
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            SiteType siteType = JSON.parseObject(condition, SiteType.class);
            qw.eq("org_code", siteType.getOrgCode());
        }
        // 雪花id倒排序
        qw.orderByDesc("id");
        // 分页查询
        return siteTypeMapper.selectPage(page, qw);
    }

    @Override
    public void saveAndUpdate(SiteType siteType) {
        if (siteType.getId() == null) {
            siteType.setId(YitIdHelper.nextId());
            siteTypeMapper.insert(siteType);
        } else
            siteTypeMapper.updateById(siteType);
    }

}
