package com.unissoft.basic.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.basic.model.TimingTask;
import com.unissoft.basic.mapper.TimingTaskMapper;
import com.unissoft.basic.model.TimingTaskList;
import com.unissoft.basic.model.TimingTaskReq;
import com.unissoft.basic.service.TimingTaskService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.*;
import com.unissoft.deviceManage.mapper.DeviceInfoMapper;
import com.unissoft.deviceManage.model.CheckParams;
import com.unissoft.deviceManage.model.DeviceInfo;
import com.unissoft.enums.BeanEnum;
import com.unissoft.enums.SysJobStatus;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 定时任务设置表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-24
 */
@Service
public class TimingTaskServiceImpl extends ServiceImpl<TimingTaskMapper, TimingTask> implements TimingTaskService {
    @Resource
    private TimingTaskMapper timingTaskMapper;
    @Resource
    private DeviceInfoMapper deviceInfoMapper;
    @Resource
    private CronTaskRegistrar cronTaskRegistrar;

    @Override
    public IPage<TimingTask> getPageList(PageParam pageParam) {
        // 创建分页条件对象
        IPage<TimingTask> page = new Page<>(pageParam.getPageIndex(), pageParam.getPageSize());
        // 查询对象
        QueryWrapper<TimingTask> qw = new QueryWrapper<>();
        // 解析查询条件
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            TimingTask timingTask = JSON.parseObject(condition, TimingTask.class);
            qw.eq("org_code", timingTask.getOrgCode());
        }
        return timingTaskMapper.selectPage(page, qw);
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void saveList(TimingTaskList timingTaskList) {
        // 保存
        if (Objects.isNull(timingTaskList.getId()))
            timingTaskList.setId(YitIdHelper.nextId());
        timingTaskList.setBuildTime(new Date());
        // 字符串拼接
        joiningString(timingTaskList);
        int insert = timingTaskMapper.insert(timingTaskList);
        // 新增定时任务
        if (insert > 0) {
            if (timingTaskList.getOnOrOff().equals(SysJobStatus.NORMAL.ordinal())) {
                // beanName: 执行逻辑类,methodName类方法,param: 设备编码+通讯参数(待定)
                SchedulingRunnable task = new SchedulingRunnable(
                        // TODO param WAITED
                        timingTaskList.getBeanName(), timingTaskList.getMethodName(), timingTaskList.getMethodParams()
                );
                cronTaskRegistrar.addCronTask(task, timingTaskList.getDetectionCron());
            }
        }

    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void updateList(TimingTaskList timingTaskList) {
        /* 字符串拼接 */
        joiningString(timingTaskList);
        // 更新
        timingTaskList.setBuildTime(new Date());
        int i = timingTaskMapper.updateById(timingTaskList);
        // 修改定时任务,先移除原来的任务,再启动新任务
        if (i > 0) {
            // 先移除
            SchedulingRunnable task = new SchedulingRunnable(timingTaskList.getBeanName(), timingTaskList.getMethodName(), timingTaskList.getMethodParams());
            cronTaskRegistrar.removeCronTask(task);
            // 再添加
            if (timingTaskList.getOnOrOff().equals(SysJobStatus.NORMAL.ordinal())) {
                SchedulingRunnable taskAdd = new SchedulingRunnable(timingTaskList.getBeanName(), timingTaskList.getMethodName(), timingTaskList.getMethodParams());
                cronTaskRegistrar.addCronTask(taskAdd, timingTaskList.getDetectionCron());
            }
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean deleteById(Long id) {
        // 先查询获取移除定时任务参数
        TimingTask timingTask = timingTaskMapper.selectById(id);
        // 删除
        int i = timingTaskMapper.deleteById(id);
        // 移除定时任务
        if (i > 0) {
            SchedulingRunnable task = new SchedulingRunnable(timingTask.getBeanName(), timingTask.getMethodName(), timingTask.getMethodParams());
            cronTaskRegistrar.removeCronTask(task);
            return true;
        }
        return false;
    }

    @Override
    @SuppressWarnings("all")
    public TimingTaskList getByIdList(Long id) {
        // 返回子类对象
        TimingTaskList timingTaskList = new TimingTaskList();
        // 查询主信息
        TimingTask timingTask = timingTaskMapper.selectById(id);
        if (Objects.nonNull(timingTask)) {
            // 查询设备信息
            List<DeviceInfo> list = deviceInfoMapper.selectInEqCodeList(
                    timingTask.getEqCode().split(","), timingTask.getOrgCode(), timingTask.getLabelCode()
            );
            // 父类转子类
            BeanUtils.copyProperties(timingTask, timingTaskList);
            // 添加设备信息
            if (!CollectionUtils.isEmpty(list))
                timingTaskList.setDevInfoList(list);
        }
        return timingTaskList;
    }

    private synchronized void joiningString(TimingTaskList timingTaskList) {
        // 获取设备信息
        List<DeviceInfo> devInfoList = timingTaskList.getDevInfoList();
        if (!CollectionUtils.isEmpty(devInfoList)) {
            // 拼接eqCode
            DeviceInfo deviceInfo = devInfoList.get(0);
            String eqCodes;
            if (deviceInfo.getEqCode().contains(","))
                eqCodes = devInfoList.stream().map(DeviceInfo::getEqCode).collect(Collectors.joining(""));
            else
                eqCodes = devInfoList.stream().map(DeviceInfo::getEqCode).collect(Collectors.joining(","));
            timingTaskList.setEqCode(eqCodes);
            // 获取产品编码转换bean
            String pcc = timingTaskList.getProductCategoryCode();
            String labelCode = deviceInfo.getLabelCode();
            BeanEnum beanEnum = ConvertBeanName.convertMethod(pcc);
            // 获取beanName
            timingTaskList.setBeanName(beanEnum.getBeanName());
            // 获取methodName
            timingTaskList.setMethodName(beanEnum.getMethodName());
            // 定时任务参数
            String params = timingTaskList.getId().toString();
            timingTaskList.setMethodParams(params);
            // 设备数量
            timingTaskList.setEqSum(devInfoList.size());
            timingTaskList.setProductCategoryCode(pcc);
            timingTaskList.setLabelCode(labelCode);
        }
    }


    @Override
    public Long saveTask(TimingTaskReq timingTaskReq) {
        // 保存数据转换
        TimingTaskList task = new TimingTaskList();
        long id = YitIdHelper.nextId();
        task.setId(id);
        this.outSetDataJoin(timingTaskReq, task);
        /*BeanUtils.copyProperties(timingTaskReq, task);
        // 查询其它信息
        QueryWrapper<DeviceInfo> qw = new QueryWrapper<>();
        // 组织机构编码
        String orgCode = timingTaskReq.getOrgCode();
        // 设备标签编码(平台仓房编码)
        String labelCode = timingTaskReq.getLabelCode();
        // 产品编码
        String productCategoryCode = timingTaskReq.getProductCategoryCode();
        if (!StringUtils.isEmpty(orgCode))
            qw.eq("org_code", orgCode);
        if (!StringUtils.isEmpty(orgCode))
            qw.eq("label_code", labelCode);
        if (!StringUtils.isEmpty(orgCode))
            qw.eq("pro_code", productCategoryCode);
        task.setDevInfoList(deviceInfoMapper.selectList(qw));*/
        // 调用保存
        this.saveList(task);
        return id;
    }

    @Override
    public void updateTask(TimingTaskReq timingTaskReq) {
        // 保存数据转换
        TimingTaskList task = new TimingTaskList();
        this.outSetDataJoin(timingTaskReq, task);
        this.updateList(task);
    }

    @Override
    public CheckParams executorTimingParam(Long id) {
        return timingTaskMapper.executorTimingParam(id);
    }

    @Override
    public List<Map<String, Object>> getByTask(String orgCode, String eqCode) {
        eqCode = String.format("%s%s%s", "%", eqCode, "%");
        return timingTaskMapper.getByTask(orgCode, eqCode);
    }

    private void outSetDataJoin(TimingTaskReq timingTaskReq, TimingTaskList task) {
        BeanUtils.copyProperties(timingTaskReq, task);
        // 查询其它信息
        QueryWrapper<DeviceInfo> qw = new QueryWrapper<>();
        // 组织机构编码
        String orgCode = timingTaskReq.getOrgCode();
        // 设备标签编码(平台仓房编码)
        String labelCode = timingTaskReq.getLabelCode();
        // 产品编码
        String productCategoryCode = timingTaskReq.getProductCategoryCode();
        if (!StringUtils.isEmpty(orgCode))
            qw.eq("org_code", orgCode);
        if (!StringUtils.isEmpty(labelCode))
            qw.eq("label_code", labelCode);
        if (!StringUtils.isEmpty(productCategoryCode))
            qw.eq("pro_code", productCategoryCode);
        task.setDevInfoList(deviceInfoMapper.selectList(qw));
    }

}