package com.unissoft.config;

import com.alibaba.fastjson.JSON;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.DateUtils;
import com.unissoft.productManage.service.ProductInfoService;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.service.UserInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author qingsong.han
 * @description: 需要初始化数据的
 * @create 2022-10-31 18:06
 */
@Slf4j
@Component
@Order(1)
public class InitBasic implements ApplicationRunner {
    @Value("${spring.application.name}")
    private String appName;
    @Resource
    private ProductInfoService productInfoService;
    @Resource
    private UserInfoService userInfoService;
    // 初始化用户
    private static final String userInit = "{\"userName\":\"admin\",\"userRealName\":\"超级管理员\",\"userPassword\":\"78e8b458042a9b7c\",\"userState\":\"1\",\"orgCode\": \"001\",\"orgName\":\"初始化\"}";

    /**
     * 初始化入口
     *
     * @param args 上下文参数
     * @throws Exception null
     */
    @Override
    public void run(ApplicationArguments args) throws Exception {
        log.info("init {} data run!", appName);
        this.initUserAdmin();
        this.orgCodeKnowProCode();

    }

    /**
     * order 1
     * 初始化admin用户
     */
    private void initUserAdmin() {
        if (userInfoService.count() == 0) {
            UserInfo userInfo = JSON.parseObject(userInit, UserInfo.class);
            userInfo.setId(YitIdHelper.nextId());
            userInfo.setBuildTime(DateUtils.getDate());
            userInfoService.save(userInfo);
            log.info("init user admin success!user: admin,password: 123456");
        }
    }

    /**
     * order 2
     * 初始化组织机构和产品编码对应关系到redis,每次重启模块执行
     * <p>
     * 手动刷新redis组织机构编码和产品编码缓存数据
     * http://localhost:8071/productInfo/redisX1ORG?orgCode=307308681810054
     * 参数orgCode
     * 多个proCode逗号隔开的数据
     * orgCode:proCode,...
     * proCode:orgCode
     */
    private void orgCodeKnowProCode() {
        productInfoService.orgCodeKnowProCode(null);
        log.info("init orgCode to proCode and convert init success in the redis key of REDIS_X1_ORG!");
    }

}
