package com.unissoft.config;

import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;

/**
 * @author qingsong.han
 * @description: 配置Mqtt公用部分, 主要是提供Mqtt客户端Bean, mqtt消息处理配置
 * @create 2022-06-14 11:10
 */
@Configuration
@SuppressWarnings("all")
@ConditionalOnProperty(value = "mqtt.enabled", havingValue = "true")
public class MqttBaseConfig {
    /**
     * 出站-生产者
     */
    public static final String CHANNEL_NAME_OUT = "mqttOutputChannel";

    /**
     * 入站-消费者
     */
    static final String CHANNEL_NAME_IN = "mqttInputChannel";

    @Value("${mqtt.serverURIs}")
    private String mqttServerURIs;
    @Value("${mqtt.username}")
    private String mqttUserName;
    @Value("${mqtt.password}")
    private String mqttPwd;
    @Value("${mqtt.keepAliveInterval}")
    private Integer keepAliveInterval;
    @Value("${mqtt.cleanSession}")
    private Boolean cleanSession;

    // 客户端与服务器之间的连接意外中断，服务器将发布客户端的“遗嘱”消息
    /*private static final byte[] WILL_DATA;

    static {
        WILL_DATA = "offline".getBytes();
    }*/


    /**
     * 创建设置MQTT Broker连接属性,如果使用SSL验证,也在这里设置(构造一个默认的mqtt客户端操作bean)
     *
     * @return MqttPahoClientFactory
     */
    @Bean
    public MqttPahoClientFactory factory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        MqttConnectOptions options = new MqttConnectOptions();
        //如果配置了用户密码这里需要对应设置
        options.setUserName(mqttUserName);
        options.setPassword(mqttPwd.toCharArray());
        String[] array = mqttServerURIs.split(",");
        options.setServerURIs(array);
        // 设为false，客户端掉线后 服务器端不会清除session，当重连后可以接收之前订阅主题的消息。当客户端上线后会接受到它离线的这段时间的消息
        options.setCleanSession(cleanSession);
        // 设置会话心跳时间 单位为秒
        options.setKeepAliveInterval(keepAliveInterval);
        // 设置"遗嘱"消息的话题,若客户端与服务器之间的连接意外中断,服务器将发布客户端的"遗嘱"消息
//        options.setWill("willTopic", WILL_DATA, 2, false);
        factory.setConnectionOptions(options);
        return factory;
    }

}
