package com.unissoft.deviceManage.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.common.SystemLog;
import com.unissoft.deviceManage.model.DeviceInfo;
import com.unissoft.deviceManage.service.DeviceInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * <p>
 * 设备信息表 前端控制器
 * </p>
 *
 * @author root
 * @since 2022-06-16
 */
@Api(tags = "设备管理")
@RestController
@RequestMapping("/device-info")
public class DeviceInfoController {
    @Resource
    private DeviceInfoService deviceInfoService;

    @PostMapping("/getPageList")
    @ApiOperation(value = "设备", notes = "分页列表")
    public ResultView getPageList(@RequestBody PageParam pageParam) {
        try {
            IPage<DeviceInfo> pageList = deviceInfoService.getPageList(pageParam);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @PostMapping("/saveOrUpdate")
    @ApiOperation(value = "设备", notes = "新增(id为空)或修改(id)")
    @SystemLog(operModul = "设备管理", operType = "设备信息", operDesc = "新增修改设备信息")
    public ResultView saveOrUpdate(@RequestBody DeviceInfo deviceInfo) {
        try {
            deviceInfoService.saveAndUpdate(deviceInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

    @ApiOperation(value = "设备", notes = "根据id查询")
    @GetMapping("/getById/{id}")
    public ResultView getById(@PathVariable Long id) {
        DeviceInfo deviceInfo = deviceInfoService.queryById(id);
        return ResultView.success(deviceInfo);
    }

    @GetMapping("/deleteById/{id}")
    @ApiOperation(value = "设备", notes = "删除消息")
    public ResultView deleteById(@PathVariable Long id) {
        return deviceInfoService.deleteById(id);
    }

}
