package com.unissoft.deviceManage.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.common.SystemLog;
import com.unissoft.deviceManage.model.DeviceLabel;
import com.unissoft.deviceManage.service.DeviceLabelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;

/**
 * <p>
 * 设备标签信息表 前端控制器
 * </p>
 *
 * @author root-设备标签
 * @since 2022-06-16
 */
@Api(tags = "设备管理")
@RestController
@RequestMapping("/device-label")
public class DeviceLabelController {

    @Resource
    private DeviceLabelService deviceLabelService;

    @PostMapping("/getPageList")
    @ApiOperation(value = "设备标签", notes = "分页列表")
    public ResultView getPageList(@RequestBody PageParam pageParam) {
        try {
            IPage<DeviceLabel> pageList = deviceLabelService.getPageList(pageParam);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @PostMapping("/saveOrUpdate")
    @ApiOperation(value = "设备标签", notes = "新增(id为空)或修改(id)")
    @SystemLog(operModul = "设备管理", operType = "设备标签", operDesc = "新增修改设备标签")
    public ResultView saveOrUpdate(@RequestBody DeviceLabel deviceLabel) {
        try {
            return ResultView.success(deviceLabelService.saveAndUpdate(deviceLabel));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @ApiOperation(value = "设备标签", notes = "根据id查询")
    @GetMapping("/getById/{id}")
    public ResultView getById(@PathVariable Long id) {
        DeviceLabel label = deviceLabelService.getById(id);
        return ResultView.success(label);
    }

    @ApiOperation(value = "设备标签", notes = "根据orgCode和labelCode查询对象")
    @GetMapping("/getByObjectForOnlyData/{orgCode}/{labelCode}")
    public ResultView getByObjectForOnlyData(@PathVariable String orgCode, @PathVariable String labelCode) {
        DeviceLabel label = deviceLabelService.getByObjectForOnlyData(orgCode, labelCode);
        return ResultView.success(label);
    }

    @GetMapping("/deleteById/{id}")
    @ApiOperation(value = "设备标签", notes = "根据id删除")
    @SystemLog(operModul = "设备管理", operType = "设备标签", operDesc = "删除信息")
    public ResultView deleteById(@PathVariable Long id) {
        return deviceLabelService.deleteById(id);
    }

    @GetMapping("/deleteByObjectForOnlyData/{orgCode}/{labelCode}")
    @ApiOperation(value = "设备标签", notes = "根据orgCode和labelCode删除")
    @SystemLog(operModul = "设备管理", operType = "设备标签", operDesc = "删除信息")
    public ResultView deleteByObjectForOnlyData(@PathVariable String orgCode, @PathVariable String labelCode) {
        return deviceLabelService.deleteByObjectForOnlyData(orgCode, labelCode);
    }

}
