package com.unissoft.deviceManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.deviceManage.mapper.DeviceLabelMapper;
import com.unissoft.deviceManage.model.DeviceLabel;
import com.unissoft.deviceManage.service.DeviceLabelService;
import com.unissoft.productManage.model.ProductCategory;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;

/**
 * <p>
 * 设备标签信息表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-16
 */
@Slf4j
@Service
public class DeviceLabelServiceImpl extends ServiceImpl<DeviceLabelMapper, DeviceLabel> implements DeviceLabelService {
    @Resource
    private DeviceLabelMapper deviceLabelMapper;

    @Override
    public IPage<DeviceLabel> getPageList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        String condition = pageParam.getCondition();
        QueryWrapper<DeviceLabel> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(condition)) {
            com.alibaba.fastjson.JSONObject jsStr = JSON.parseObject(condition);
            String orgCode = jsStr.getString("org_code");//产品id
            if (!StringUtils.isEmpty(orgCode)) {
                queryWrapper.eq("org_code", orgCode);
            }
        }
        queryWrapper.orderByDesc("update_time");
        // 分页对象
        IPage<DeviceLabel> page = new Page<>(pageIndex, pageSize);
        return deviceLabelMapper.selectPage(page, queryWrapper);
    }

    @Override
    public long saveAndUpdate(DeviceLabel deviceLabel) {
        long id = -1;
        if (deviceLabel.getId() == null) {
            id = YitIdHelper.nextId();
            deviceLabel.setId(id);
            deviceLabel.setUpdateTime(DateUtils.getDate());
            if (StringUtils.isEmpty(deviceLabel.getLabelCode()))
                deviceLabel.setLabelCode(String.valueOf(YitIdHelper.nextId()));
            deviceLabelMapper.insert(deviceLabel);
        } else {
            deviceLabelMapper.updateById(deviceLabel);
        }
        return id;
    }

    @Override
    public ResultView deleteById(Long id) {
        //根据id查询此条数据是否可被删除
        DeviceLabel deviceLabel = deviceLabelMapper.selectById(id);
        if (deviceLabel != null) {
            if (deviceLabel.getDelFlag() != 1) {//可以删除
                int a = deviceLabelMapper.deleteById(id);//删除
                if (a > 0) {
                    return ResultView.success();
                }
            } else {//不可以删除
                return ResultView.error(MyConstant.NOT_DEL);
            }
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @Override
    public DeviceLabel getByObjectForOnlyData(String orgCode, String labelCode) {
        return deviceLabelMapper.selectOne(this.queryLike(orgCode, labelCode));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultView deleteByObjectForOnlyData(String orgCode, String labelCode) {
        try {
            int deleteFlag = deviceLabelMapper.delete(this.queryLike(orgCode, labelCode));
            if (deleteFlag == 1)
                ResultView.success();
            else
                ResultView.error(MyConstant.ID_NULL);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    private synchronized QueryWrapper<DeviceLabel> queryLike(String orgCode, String labelCode) {
        QueryWrapper<DeviceLabel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_code", orgCode);
        queryWrapper.eq("label_code", labelCode);
        return queryWrapper;
    }

}
