package com.unissoft.minio.controller;

import com.unissoft.common.MyConstant;
import com.unissoft.common.ResultView;
import com.unissoft.utils.MinioUtils;

import io.minio.errors.MinioException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * 文件描述
 *
 * @author tongxi.xia
 * @date 2020年12月04日 10:18
 */
@Api(tags = "文件服务控制器")
@RestController
@RequestMapping("/minio")
public class MinioController {

    Logger log = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private MinioUtils minioUtils;

    /**
     * 上传文件
     */
    @ApiOperation(value = "图片上传", notes = "")
    @PostMapping("/testupload")
    public ResultView upload(@RequestParam(name = "file", required = false) MultipartFile file) throws Exception {
        String str = minioUtils.uploadFile(file, MyConstant.BUCKET_NAME);
        return ResultView.success(str);
    }

    /**
     * 删除文件
     *
     * @param objectName
     * @throws Exception
     */
    @ApiOperation(value = "删除图片", notes = "")
    @DeleteMapping("/delete/{objectName}")
    public ResultView delete(@PathVariable("objectName") String objectName) throws Exception {
        minioUtils.removeObject(MyConstant.BUCKET_NAME, objectName);
        return ResultView.success("删除成功");
    }

    /**
     * 下载文件到本地
     *
     * @param objectName
     * @param response
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "下载图片", notes = "")
    @GetMapping("/download/{objectName}")
    public ResponseEntity download2(@PathVariable("objectName") String objectName, HttpServletResponse response) throws Exception {
        ResponseEntity<byte[]> responseEntity = null;
        InputStream stream = null;
        ByteArrayOutputStream output = null;
        try {
            // 获取"myobject"的输入流。
            stream = minioUtils.getObject(MyConstant.BUCKET_NAME, objectName);
            if (stream == null) {
                System.out.println("文件不存在");
            }
            //用于转换byte
            output = new ByteArrayOutputStream();
            byte[] buffer = new byte[4096];
            int n = 0;
            while (-1 != (n = stream.read(buffer))) {
                output.write(buffer, 0, n);
            }
            byte[] bytes = output.toByteArray();

            //设置header
            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.add("Accept-Ranges", "bytes");
            httpHeaders.add("Content-Length", bytes.length + "");
//            objectName = new String(objectName.getBytes("UTF-8"), "ISO8859-1");
            //把文件名按UTF-8取出并按ISO8859-1编码，保证弹出窗口中的文件名中文不乱码，中文不要太多，最多支持17个中文，因为header有150个字节限制。
            httpHeaders.add("Content-disposition", "attachment; filename=" + objectName);
            httpHeaders.add("Content-Type", "text/plain;charset=utf-8");
//            httpHeaders.add("Content-Type", "image/jpeg");
            responseEntity = new ResponseEntity<byte[]>(bytes, httpHeaders, HttpStatus.CREATED);

        } catch (MinioException e) {
            e.printStackTrace();
        } finally {
            if (stream != null) {
                stream.close();
            }
            if (output != null) {
                output.close();
            }
        }
        return responseEntity;
    }

    /**
     * 在浏览器预览图片
     *
     * @param objectName
     * @param response
     * @throws Exception
     */
    @ApiOperation(value = "在浏览器预览图片", notes = "")
    @GetMapping("/preViewPicture/{objectName}")
    public void download1(@PathVariable("objectName") String objectName, HttpServletResponse response) {
        response.setContentType("image/jpeg");
        ServletOutputStream out = null;
        ByteArrayOutputStream output = null;
        InputStream stream = null;
        try {
            out = response.getOutputStream();
            stream = minioUtils.getObject(MyConstant.BUCKET_NAME, objectName);
            output = new ByteArrayOutputStream();
            byte[] buffer = new byte[4096];
            int n = 0;
            while (-1 != (n = stream.read(buffer))) {
                output.write(buffer, 0, n);
            }
            byte[] bytes = output.toByteArray();
            out.write(bytes == null ? new byte[0] : bytes);
            out.flush();
        } catch (Exception e) {
            log.error("浏览器预览图片失败，文件名称:" + objectName + "; 报错信息：" + e.getMessage());
        } finally {
            try {
                if (stream != null) {
                    stream.close();
                }
                if (output != null) {
                    output.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                log.error("浏览器预览图片关闭资源失败" + e.getMessage());
            }
        }
    }
}

