package com.unissoft.mqtt.controller;

import com.alibaba.fastjson.JSON;
import com.unissoft.common.ResultView;
import com.unissoft.mqtt.model.MqttParams;
import com.unissoft.mqtt.outbound.MqttMessageSender;
import com.unissoft.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @author qingsong.han
 * @description: 消息测试
 * @create 2022-06-14 13:49
 */
@Slf4j
@RestController
@ConditionalOnProperty(value = "mqtt.enabled", havingValue = "true")
@RequestMapping("/mqtt")
public class MqttController {

    private final MqttMessageSender mqttMessageSender;

    @Autowired
    public MqttController(MqttMessageSender mqttMessageSender) {
        this.mqttMessageSender = mqttMessageSender;
    }

    /**
     * TODO 测试发送mqtt消息
     *
     * @param params 消息内容
     * @return ResultView
     */
    @PostMapping("/sendMessage")
    public ResultView send(@RequestBody MqttParams params) {
        try {
            this.mqttMessageSender.sendToMqtt(params.getTopic(), params.getQos(), params.getPayload());
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        log.info("发送mqtt消息完成: {}", JSON.toJSONString(params));
        return ResultView.success();
    }

    /**
     * TODO
     * redis 测试hash 保存
     * http://localhost:8071/mqtt/setRedisHash?key=REDIS_X1_ORG&item=001&value=X1XXXQ2206090069&overTime=6000
     */
    @Resource
    private RedisUtil redisUtil;

    @GetMapping("/setRedisHash")
    public void setRedisHash(@RequestParam String key, @RequestParam String item, @RequestParam Object value, @RequestParam long overTime) {
        if (overTime > 0)
            redisUtil.hset(key, item, value, overTime);
        else
            redisUtil.hset(key, item, value);
    }

    @GetMapping("/getRedisKey")
    public void getRedisKey(@RequestParam String key, @RequestParam String item, @RequestParam String typeData) {
        switch (typeData) {
            case "hash":
                redisUtil.hget(key, item);
                break;
            case "key":
                redisUtil.get(key);
                break;
            default:
                redisUtil.get(key);
                break;
        }
    }

}
