package com.unissoft.mqtt.inbound;

import com.alibaba.fastjson.JSON;
import com.unissoft.mqtt.service.MqttMessageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.integration.mqtt.support.MqttHeaders;
import org.springframework.lang.NonNull;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageHandler;
import org.springframework.messaging.MessagingException;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author qingsong.han
 * @description: mqtt订阅消息处理
 * @create 2022-06-14 11:21
 */
@Slf4j
@Component
@ConditionalOnProperty(value = "mqtt.enabled", havingValue = "true")
public class MqttMessageReceiver implements MessageHandler {
    @Resource
    private MqttMessageService mqttMessageService;

    /**
     * {
     * "headers": {
     * "mqtt_receivedRetained": false,
     * "mqtt_id": 23,
     * "mqtt_duplicate": false,
     * "id": "f68a18f9-5f15-7aa8-d327-f9e448255c64",
     * "mqtt_receivedTopic": "/PUSH/X1XXXQ2206090069/DPUT/alarm",
     * "mqtt_receivedQos": 1,
     * "timestamp": 1661766248867
     * },
     * "payload": {
     * "time": "1970-03-22 21:30:29",
     * "data": [{
     * "deviceAlarm": "306943820628102",
     * "type": "1",
     * "tempValue": "24"
     * }]
     * }
     * }
     *
     * @param message 消息
     * @throws MessagingException
     */
    @Override
    public void handleMessage(@NonNull Message<?> message) throws MessagingException {
        log.info("接收到 mqtt消息: {}", JSON.toJSONString(message));
        // 8951设备没有报警功能
        /*String topic = String.valueOf(message.getHeaders().get(MqttHeaders.RECEIVED_TOPIC));
        if (topic.contains("/DPUT/alarm")) // 报警数据
            log.info("mqtt报警消息!: {}", topic);
        else if (topic.contains("/DPUT/upload")) // 温湿度,露点数据
            mqttMessageService.uploadData(message);*/
        mqttMessageService.uploadData(message);
    }

}
