package com.unissoft.mqtt.service.impl;

import com.alibaba.fastjson.JSON;
import com.unissoft.alarm.service.AlarmRecordService;
import com.unissoft.deviceManage.model.DeviceDataPoint;
import com.unissoft.mqtt.model.MqttPayload;
import com.unissoft.mqtt.service.MqttMessageService;
import com.unissoft.utils.RedisUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author qingsong.han
 * @description:
 * @create 2022-08-22 17:41
 */
@Service
public class MqttMessageServiceImpl implements MqttMessageService {

    @Resource
    private RedisUtil redisUtil;
    @Resource
    private AlarmRecordService alarmRecordService;
    @Value("${spring.redis.overdueTime}")
    private long redisOverdueTime;

    @Override
    public void uploadData(Message<?> message) {
        /*
         * message内容
         * topic 可以设置为识别标识 例如:机构编码唯一标识
         */
//        String topic = String.valueOf(message.getHeaders().get(MqttHeaders.RECEIVED_TOPIC));
//        String headers = String.valueOf(message.getHeaders());
//        log.info("headers: {}", headers);
        /*
         *  网关上行数据格式
         *  time:数据时间
         *  deviceAlarm: 设备编码
         *  数据:温度值,湿度值,露点值
         * X1/c8951:网关数据json
         * payload:
         * {
         * 	"time": "2020-07-07 11:07:50",
         * 	"data": [{
         * 		"deviceAlarm": "306943820628102",
         * 		"tempValue": "34.560000",
         * 		"wetValue": "60.5",
         * 		"dewValue": "16.54"
         *        }, {
         * 		"deviceAlarm": "307037305270406",
         * 		"tempValue": "34.560000",
         * 		"wetValue": "60.5",
         * 		"dewValue": "16.54"
         *    }]
         * }
         */
        String payload = String.valueOf(message.getPayload());
        // log.info("payload: {}", payload);
        // headers
        Map<String, Object> mqttHeaders = message.getHeaders();
        // 数据刷新频繁,防止系统崩溃使用try catch
        try {
            // topic 获取订阅x1主题
            String subscribeTopic = (String) mqttHeaders.get("mqtt_receivedTopic");
            if (!StringUtils.isEmpty(subscribeTopic) && subscribeTopic.contains("/")) {
                String[] reTopic = subscribeTopic.split("/");
                // payload
                MqttPayload mqttPayload = JSON.parseObject(payload, MqttPayload.class);
                // 获取点位参数
                List<DeviceDataPoint> deviceDataPoints = mqttPayload.getData();
                /*
                 * redis hash redis缓存,key:x1网关SN编码,item:设备编码,value:可直接持久化到数据库的数据,time:redis数据过期时间.
                 * reTopic[2]:网关SN编码
                 * deviceAlarm:下位机识别编码或设备编码(温度点)
                 * mqttPayload:payload
                 * redisOverdueTime:过期时间
                 */
                deviceDataPoints.forEach(devData -> redisUtil.hset(reTopic[2], devData.getDevice(), devData, redisOverdueTime));
                // TODO 预警阀值处理(目前X1设备预警阀值设置是单数据类型返回,不好处理,数据量太大,可用于实时数据展示)
                alarmRecordService.saveAlarmRecord(reTopic[2], deviceDataPoints);
            } else {
                // 自定义异常
                throw new IllegalArgumentException("MQTT topic 解析错误");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
