package com.unissoft.productManage.controller;

import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.ResultView;
import com.unissoft.productManage.model.ProductCategory;
import com.unissoft.productManage.model.ProductCategoryChildren;
import com.unissoft.productManage.service.ProductCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 产品分类表 前端控制器
 * </p>
 *
 * @author root
 * @since 2022-06-17
 */
@Api(tags = "产品管理-产品分类")
@RestController
@RequestMapping("/productCategory")
public class ProductCategoryController {

    @Resource
    private ProductCategoryService categoryService;

    @GetMapping("/getTreeList/{orgCode}")
    @ApiOperation(value = "产品分类", notes = "树列表")
    public ResultView getTreeList(@PathVariable String orgCode) {
        try {
            List<ProductCategoryChildren> pageList = categoryService.getTreeList(orgCode);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }
    
    
    /**
     * 新增
     * @param productCategory
     * @return
     */
    @PostMapping("/save")
    @ApiOperation(value = "产品分类", notes = "保存")
    public ResultView save(@RequestBody ProductCategory productCategory) {
    	productCategory.setId(YitIdHelper.nextId());
    	productCategory.setCreateTime(new Date());
    	//删除标志
    	productCategory.setDelFlag(0);//可以删除
        boolean save = categoryService.save(productCategory);
        if (save)
            return ResultView.success();
        else
            return ResultView.error();
    }
    
    
    /**
     * 修改
     * @param productCategory
     * @return
     */
    @PostMapping("/update")
    @ApiOperation(value = "产品分类", notes = "修改")
    public ResultView update(@RequestBody ProductCategory productCategory) {
    	boolean b = categoryService.updateById(productCategory);
        if (b)
            return ResultView.success();
        else
            return ResultView.error();
    }
    
    /**
     * 根据主键删除
     * @param id
     * @return
     */
    @GetMapping("/deleteById/{id}")
    @ApiOperation(value = "产品分类", notes = "删除")
    public ResultView deleteById(@PathVariable Long id) {
    	
    	return categoryService.deleteById(id);
    }
    
    
    /**
     * 获取产品分类的子级
     * @return
     */
    @GetMapping("/getSonList/{orgCode}")
    @ApiOperation(value = "获取产品分类的子级", notes = "获取产品分类的子级")
    public ResultView getSonList(@PathVariable String orgCode) {
        try {
            List<ProductCategory> list = categoryService.getSonList(orgCode);
            return ResultView.success(list);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

}
