package com.unissoft.productManage.controller;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.unissoft.deviceManage.model.DeviceData;
import com.unissoft.deviceManage.model.DeviceInfo;
import com.unissoft.deviceManage.model.DeviceInfoJoinProduct;
import com.unissoft.deviceManage.model.DeviceRunData;
import com.unissoft.deviceManage.service.DeviceDataService;
import com.unissoft.deviceManage.service.DeviceInfoService;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.productManage.model.ProductInfo;
import com.unissoft.productManage.service.ProductInfoService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * <p>
 * 产品表 前端控制器
 * </p>
 *
 * @author root
 * @since 2022-06-17
 */
@Api(tags = "产品管理-产品")
@RestController
@RequestMapping("/productInfo")
public class ProductInfoController {

    @Resource
    private ProductInfoService productInfoService;
    @Resource
    private DeviceDataService deviceDataService;
    @Resource
    private DeviceInfoService deviceInfoService;

    /**
     * 新增
     *
     * @param productInfo 产品分类信息对象
     * @return
     */
    @PostMapping("/save")
    @ApiOperation(value = "产品-保存", notes = "保存")
    public ResultView save(@RequestBody ProductInfo productInfo) {
        try {
            productInfoService.saveData(productInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

    /**
     * 修改
     *
     * @param productInfo
     * @return
     * @throws Exception
     */
    @PostMapping("/update")
    @ApiOperation(value = "产品-修改", notes = "修改")
    public ResultView update(@RequestBody ProductInfo productInfo) {
        try {
            productInfoService.updateData(productInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
        return ResultView.success();
    }

    /**
     * 根据id查询
     *
     * @param id
     * @return
     */
    @GetMapping("/getById/{id}")
    @ApiOperation(value = "产品-根据id查询", notes = "根据id查询")
    public ResultView getById(@PathVariable Long id) {
        ProductInfo productInfo = productInfoService.queryById(id);
        return ResultView.success(productInfo);
    }

    /**
     * 列表查询
     *
     * @param pageParam
     * @return
     */
    @PostMapping("/getListPage")
    @ApiOperation(value = "产品分页条件查询", notes = "提交参数：{\n" +
            "  \"condition\":\"{'pro_code':'','pro_name':'','org_code':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    public ResultView getUserPage(@RequestBody PageParam pageParam) {
        IPage<ProductInfo> page = productInfoService.getListPage(pageParam);
        return ResultView.success(page);
    }

    @GetMapping("/deleteById/{id}")
    @ApiOperation(value = "产品-根据id删除", notes = "根据id删除")
    public ResultView deleteById(@PathVariable Long id) {
        return productInfoService.deleteById(id);
    }

    /**
     * 手动刷新redis组织机构编码和产品编码缓存数据 orgCode:proCode
     * http://localhost:8071/productInfo/redisX1ORG
     * 如果redis缓存数据与数据库不一致的情况,可以使用使用本接口手动同步刷新
     *
     * @return 刷新后的redis缓存数据
     */
    @GetMapping("/redisX1ORG")
    @ApiOperation(value = "产品", notes = "设置缓存(orgCode,proCode,storeHouse)")
    public ResultView redisX1ORG(String orgCode) {
        return ResultView.success(productInfoService.orgCodeKnowProCode(orgCode));
    }

    @PostMapping("/getDeviceRunDataPageList")
    @ApiOperation(value = "产品-运行数据分页列表", notes = "{\"condition\":\"{\\\"orgCode\\\": \\\"307308681810054\\\", \\\"proCode\\\": \\\"c89512024085\\\"}\",\"pageIndex\":1,\"pageSize\":10}")
    public ResultView getPageList(@RequestBody PageParam pageParam) {
        try {
            IPage<DeviceData> pageList = deviceDataService.getPageList(pageParam);
            return ResultView.success(pageList);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @GetMapping("/getDeviceRunData/{id}")
    @ApiOperation(value = "产品-运行数据-查看详情", notes = "查看详情")
    public ResultView getPageList(@PathVariable Long id) {
        try {
            List<DeviceRunData> runDatas = deviceDataService.getDeviceRunData(id);
            return ResultView.success(runDatas);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    /**
     * 产品-取消关联
     * @param id 主键
     */
    @GetMapping("/disassociateProduct/{id}")
    @ApiOperation(value = "产品-设备信息-取消关联", notes = "设备信息-取消关联")
    public ResultView disassociateProduct(@PathVariable Long id) {
        try {
            deviceInfoService.disassociateProduct(id);
            return ResultView.success();
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    /**
     * 产品-添加关联
     * @param deviceInfoJoinProduct 需要添加的设备信息集
     */
    @PostMapping("/addAssociationsProduct")
    @ApiOperation(value = "产品-设备信息-添加关联", notes = "设备信息-添加关联")
    public ResultView addAssociationsProduct(@RequestBody DeviceInfoJoinProduct deviceInfoJoinProduct) {
        try {
            deviceInfoService.addAssociationsProduct(deviceInfoJoinProduct);
            return ResultView.success();
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

}
