package com.unissoft.productManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.MyConstant;
import com.unissoft.common.ResultView;
import com.unissoft.productManage.mapper.ProductCategoryMapper;
import com.unissoft.productManage.model.ProductCategory;
import com.unissoft.productManage.model.ProductCategoryChildren;
import com.unissoft.productManage.model.ProductInfo;
import com.unissoft.productManage.service.ProductCategoryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 产品分类表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-17
 */
@Slf4j
@Service
public class ProductCategoryServiceImpl extends ServiceImpl<ProductCategoryMapper, ProductCategory> implements ProductCategoryService {

    @Resource
    private ProductCategoryMapper categoryMapper;

    @Override
    public List<ProductCategoryChildren> getTreeList(String orgCode) {
        // 查询该单位下所有数据
    	QueryWrapper<ProductCategory> queryWrapper = new QueryWrapper<ProductCategory>();
    	if(!StringUtils.isEmpty(orgCode)){
    		queryWrapper.eq("org_code", orgCode.trim());
    	}
        List<ProductCategory> productCategories = categoryMapper.selectList(queryWrapper);
        
        
		
        // 初始化父级为顶级
        Long parentId = 0L;
        // 获取最顶级父id
        Optional<ProductCategory> min = productCategories.stream().min(Comparator.comparingLong(ProductCategory::getParentId));
        if (min.isPresent())
            parentId = min.get().getParentId();
        // 转换list 为 treeList
        String proS = JSON.toJSONString(productCategories);
        List<ProductCategoryChildren> childrenList = JSON.parseArray(proS, ProductCategoryChildren.class);
        return buildTree(childrenList, parentId);
    }

    /**
     * 获取数据子集
     *
     * @param listParam 元数据
     * @param parentId  用于获取key 为最顶级 parentId
     * @return tree 结构 数据
     */
    private List<ProductCategoryChildren> buildTree(List<ProductCategoryChildren> listParam, Long parentId) {
        // 接收参数非空判断
        if (CollectionUtils.isEmpty(listParam)) {
            return Collections.emptyList();
        }
        // id 为key list 转map
        Map<Long, ProductCategoryChildren> idMap = listParam.parallelStream().collect(Collectors.toMap(ProductCategoryChildren::getId, p -> p));
        // 数据根据ParentId分组
        Map<Long, List<ProductCategoryChildren>> pidGroupRolesMap = listParam.parallelStream().collect(Collectors.groupingBy(ProductCategoryChildren::getParentId));
        // set children 值
        pidGroupRolesMap.forEach((k, v) -> {
            // 获取parent id key
            ProductCategoryChildren parentRole = idMap.get(k);
            if (parentRole != null) {
                parentRole.setChildren(v);
            }
        });
        return pidGroupRolesMap.get(parentId);
    }

	@Override
    @SuppressWarnings("all")
	public ResultView deleteById(Long id) {
		// TODO Auto-generated method stub
		//根据id查询此条数据是否可被删除
		ProductCategory productCategory = categoryMapper.selectById(id);
		if(productCategory != null){
			if(productCategory.getDelFlag().equals(0)){//可以删除
				int a = categoryMapper.deleteById(id);//删除
				if(a>0){
					return ResultView.success();
				}
			}else if(productCategory.getDelFlag().equals(1)){//不可以删除
				return ResultView.error(MyConstant.NOT_DEL);
			}
		}
		 
		return ResultView.error(MyConstant.ID_NULL);
		
	}

	@Override
	public List<ProductCategory> getSonList(String orgCode) {
		// TODO Auto-generated method stub
		
        QueryWrapper<ProductCategory> queryWrapper = new QueryWrapper<ProductCategory>();
        queryWrapper.ne("parent_id", 0);
        queryWrapper.eq("org_code", orgCode);
		return categoryMapper.selectList(queryWrapper);
	}

}
