package com.unissoft.productManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.productManage.mapper.ProductCategoryMapper;
import com.unissoft.productManage.mapper.ProductInfoMapper;
import com.unissoft.productManage.model.ProductCategory;
import com.unissoft.productManage.model.ProductInfo;
import com.unissoft.productManage.service.ProductInfoService;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import com.unissoft.utils.MapConvertUtils;
import com.unissoft.utils.MinioUtils;
import com.unissoft.utils.RedisUtil;
import lombok.Data;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 产品表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-17
 */
@Service
public class ProductInfoServiceImpl extends ServiceImpl<ProductInfoMapper, ProductInfo> implements ProductInfoService {

    @Resource
    private ProductInfoMapper productInfoMapper;
    @Resource
    private ProductCategoryMapper productCategoryMapper;
    @Resource
    private ProductCategoryMapper categoryMapper;
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private MinioUtils minioUtils;
    @Resource
    private MapConvertUtils mapConvertUtils;

    @Override
    public IPage<ProductInfo> getListPage(PageParam pageParam) {
        // TODO Auto-generated method stub
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        String condition = pageParam.getCondition();
        QueryWrapper<ProductInfo> queryWrapper = new QueryWrapper<>();

        if (!StringUtils.isEmpty(condition)) {

            com.alibaba.fastjson.JSONObject jsStr = JSON.parseObject(condition);
            String proCode = jsStr.getString("pro_code");//产品id
            String proName = jsStr.getString("pro_name");//产品名称
            String orgCode = jsStr.getString("org_code");//组织code
            if (!StringUtils.isEmpty(proCode)) {
                queryWrapper.eq("pi.pro_code", proCode);
            }
            if (!StringUtils.isEmpty(proName)) {
                queryWrapper.eq("pi.pro_name", proName);
            }
            if (!StringUtils.isEmpty(orgCode)) {
                queryWrapper.eq("pi.org_code", orgCode);
            }
        }
        queryWrapper.orderByDesc("pi.create_time");
        // 分页对象
        IPage<ProductInfo> page = new Page<>(pageIndex, pageSize);
        return productInfoMapper.selectPageList(page, queryWrapper);
    }

    @Override
    public ResultView deleteById(Long id) {
        //先去查询该产品的产品类别 有没有其他的产品  有则不用更改产品分类的删除标志 没有则更改
        ProductInfo productInfo = productInfoMapper.selectById(id);
        if (productInfo != null) {
            QueryWrapper<ProductInfo> queryWrapper = new QueryWrapper<ProductInfo>();
            queryWrapper.eq("pro_category_code", productInfo.getProCategoryCode());
            List<ProductInfo> proList = productInfoMapper.selectList(queryWrapper);
            if (proList.size() > 1) {//有自身的一条数据 所以大于1
                //不用处理
            } else {
                UpdateWrapper<ProductCategory> updateWrapper = new UpdateWrapper<ProductCategory>();
                updateWrapper.eq("product_category_code", productInfo.getProCategoryCode());
                updateWrapper.set("del_flag", 0);//可删除
                productCategoryMapper.update(null, updateWrapper);
            }
            productInfoMapper.deleteById(id);
            // 删除成功更新redis缓存
            orgCodeKnowProCode(productInfo.getOrgCode());
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @Override
    public ProductInfo queryById(Long id) {
        // TODO Auto-generated method stub
        return productInfoMapper.queryById(id);
    }

    @Override
    public List<Map<String, String>> orgCodeKnowProCode(String orgCode) {
        // 设置缓存,orgCode为空时刷新设置所有机构缓存(orgCode:SN1,SN2)
        List<Map<String, String>> list = productInfoMapper.orgCodeKnowProCode(orgCode);
        if (!CollectionUtils.isEmpty(list)) {
            Map<String, String> map = mapConvertUtils.groupJoin("orgCode", "proCode", list);
            map.keySet().forEach(key -> redisUtil.hset(MyConstant.REDIS_X1_ORG, key, map.get(key)));
        }
        // 设置缓存SN:orgCode
        List<Map<String, String>> list1 = productInfoMapper.proCodeKnowOrgCode(orgCode);
        list1.forEach(ml -> redisUtil.hset(MyConstant.REDIS_X1_ORG, ml.get("proCode"), ml.get("orgCode")));
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveData(ProductInfo productInfo) {
        productInfo.setId(YitIdHelper.nextId());
        productInfo.setCreateTime(new Date());
        if (productInfoMapper.insert(productInfo) > 0) {
            //更改产品分类的删除标志
            UpdateWrapper<ProductCategory> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("product_category_code", productInfo.getProCategoryCode());
            updateWrapper.set("del_flag", 1);//不可删除
            categoryMapper.update(null, updateWrapper);
            // 操作成功更新缓存
            this.orgCodeKnowProCode(productInfo.getOrgCode());
        } else {
            throw new RuntimeException("productInfo save failed");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateData(ProductInfo productInfo) throws Exception {
        // 先更新数库数据
        productInfoMapper.updateById(productInfo);
        // 在更新数据成功后在删除mini图片
        if (productInfo.getProIcons() != null) {
            ProductInfo productOld = productInfoMapper.selectById(productInfo.getId());
            if (productOld.getProIcons() != null && !productOld.getProIcons().equals(productInfo.getProIcons())) {
                //如果存在图片先删除图片
                minioUtils.removeObject(MyConstant.BUCKET_NAME, productOld.getProIcons());
            }
        }
        // 操作成功更新缓存
        this.orgCodeKnowProCode(productInfo.getOrgCode());
    }

}
