package com.unissoft.systemManage.controller;

import com.unissoft.common.ResultView;
import com.unissoft.systemManage.model.AlarmStatistics;
import com.unissoft.systemManage.model.CPUMonitor;
import com.unissoft.systemManage.model.HomePage;
import com.unissoft.systemManage.service.HomePageService;
import com.unissoft.utils.OSUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author qingsong.han
 * @description: 首页统计
 * @create 2022-06-29 10:36
 */
@Api(tags = "首页")
@RestController
@RequestMapping("/homePage")
public class HomePageController {

    @Resource
    private HomePageService homePageService;

    @GetMapping("/getHomePageUp/{orgCode}")
    @ApiOperation(value = "首页小图标", notes = "组合数据(orgCode非空)")
    public ResultView getHomePageUp(@PathVariable @NotNull String orgCode) {
        try {
            HomePage homePage = homePageService.getHomePageUp(orgCode);
            return ResultView.success(homePage);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @GetMapping("/getSystemCPURate")
    @ApiOperation(value = "首页cpu使用率", notes = "CPU使用率")
    public ResultView getCPURate() {
        try {
            CPUMonitor cpu = homePageService.getCPURate();
            return ResultView.success(cpu);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @ApiOperation(value = "报警统计", notes = "报警统计(optionTime=3:7天,4:30天)")
    @GetMapping("/getByTimeList/{orgCode}/{optionTime}")
    public ResultView getByTimeList(@PathVariable @NotNull String orgCode, @PathVariable @NotNull String optionTime) {
        try {
            List<AlarmStatistics> list = homePageService.getByTimeList(orgCode, optionTime);
            return ResultView.success(list);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @GetMapping("/testCPU")
    public int testCPU() {
        return OSUtils.cpuUsage();
    }

}
