package com.unissoft.systemManage.controller;

import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.common.ResultView;
import com.unissoft.common.SystemLog;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.OrgInfoChildren;
import com.unissoft.systemManage.service.OrgInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.List;

/**
 * <p>
 * 组织机构和仓房表 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@Api(tags = "组织机构管理")
@RestController
@RequestMapping("/orgInfo")
public class OrgInfoController {

    private OrgInfoService orgInfoService;

    @Autowired
    public void setRoleInfoController(OrgInfoService orgInfoService) {
        this.orgInfoService = orgInfoService;
    }

    @ApiOperation(value = "组织机构信息", notes = "机构id获取机构信息")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable Long id) {
        OrgInfo orgInfo = orgInfoService.getById(id);
        return ResultView.success(orgInfo);
    }

    @PostMapping("/authority/save")
    @ApiOperation(value = "组织机构信息", notes = "保存机构信息")
    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "新增组织机构")
    public ResultView save(@RequestBody OrgInfo orgInfo) {
        orgInfo.setId(YitIdHelper.nextId());
        orgInfo.setBuildTime(DateUtils.getDate());
        // 添加默认orgCode
        if (StringUtils.isEmpty(orgInfo.getOrgCode()))
            orgInfo.setOrgCode(String.valueOf(YitIdHelper.nextId()));
        boolean save = orgInfoService.save(orgInfo);
        if (save)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @PostMapping("/authority/update")
    @ApiOperation(value = "组织机构信息", notes = "更新机构信息")
    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "修改组织机构")
    public ResultView update(@RequestBody OrgInfo orgInfo) {
        orgInfo.setUpdateTime(DateUtils.getDate());
        boolean b = orgInfoService.updateById(orgInfo);
        if (b)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @GetMapping("/authority/getByIdDelete/{id}")
    @ApiOperation(value = "组织机构信息", notes = "id删除机构信息")
    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "删除组织机构")
    public ResultView getByIdDelete(@PathVariable Long id) {
        if (orgInfoService.removeById(id)) {
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    /**
     * 根据对象查询
     *
     * @param orgInfo 查询对象
     * @return list
     */
    @ApiOperation(value = "组织机构信息", notes = "获取机构信息列表-不支持分页")
    @PostMapping("/authority/getList")
    public ResultView getList(@RequestBody OrgInfo orgInfo) {
        List<OrgInfo> list = orgInfoService.getList(orgInfo);
        return ResultView.success(list);
    }

    @ApiOperation(value = "组织机构信息", notes = "获取机构树信息")
    @GetMapping("/authority/getTreeList")
    public ResultView getTreeList(@PathParam("orgName") String orgName) {
        List<OrgInfoChildren> listTree = orgInfoService.getTreeList(orgName);
        return ResultView.success(listTree);
    }

    /**
     * 返回map类型 组织机构编码
     *
     * @return map key:orgCode,value:orgName
     */
    @ApiOperation(value = "组织机构信息", notes = "获取机构map{key:orgCode,value:orgName}")
    @GetMapping("/authority/getListMap")
    public ResultView getListMap() {
        return ResultView.success(orgInfoService.getOrgMap());
    }

}
