package com.unissoft.systemManage.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.*;
import com.unissoft.systemManage.model.RoleInfo;
import com.unissoft.systemManage.service.RoleInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 用户角色表 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@Api(tags = "角色管理")
@RestController
@RequestMapping("/roleInfo")
public class RoleInfoController {

    private RoleInfoService roleInfoService;

    @Autowired
    public void setRoleInfoController(RoleInfoService roleInfoService) {
        this.roleInfoService = roleInfoService;
    }

    @GetMapping("/authority/getRoleById/{id}")
    @ApiOperation(value = "角色管理", notes = "id获取")
    public ResultView getRoleById(@PathVariable Long id) {
        RoleInfo roleInfo = roleInfoService.getById(id);
        return ResultView.success(roleInfo);
    }

    @PostMapping("/authority/getRolePage")
    @ApiOperation(value = "角色管理", notes = "分页列表")
    public ResultView getRoleInfoPage(@RequestBody PageParam pageParam) {
        IPage<RoleInfo> page = roleInfoService.getRoleInfoPage(pageParam);
        return ResultView.success(page);
    }

    @PostMapping("/authority/saveRoleInfo")
    @ApiOperation(value = "角色管理", notes = "新增保存")
    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "新增角色信息")
    public ResultView saveRoleInfo(@RequestBody RoleInfo roleInfo) {
        roleInfo.setId(YitIdHelper.nextId());
        roleInfo.setBulidTime(DateUtils.getDate());
        boolean save = roleInfoService.save(roleInfo);
        if (save)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @PostMapping("/authority/updateRoleInfo")
    @ApiOperation(value = "角色管理", notes = "更新修改")
    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "修改角色信息")
    public ResultView updateRoleInfo(@RequestBody RoleInfo roleInfo) {
        roleInfo.setBulidTime(DateUtils.getDate());
        boolean b = roleInfoService.updateById(roleInfo);
        if (b)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @GetMapping("/authority/getByIdDeleteRoleInfo/{id}")
    @ApiOperation(value = "角色管理", notes = "id删除")
    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "删除角色信息")
    public ResultView getByIdDeleteRoleInfo(@PathVariable Long id) {
        if (roleInfoService.removeById(id)) {
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

}
