package com.unissoft.systemManage.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.*;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.service.UserInfoService;
import com.unissoft.utils.IdentifyCodeUtils;
import com.unissoft.utils.JWTUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.websocket.server.PathParam;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-19
 */

@Api(tags = "用户管理")
@Slf4j
@RestController
@RequestMapping("/userInfo")
public class UserInfoController {

    private UserInfoService userInfoService;

    @Autowired
    public void setUserInfoController(UserInfoService userInfoService) {
        this.userInfoService = userInfoService;
    }

    @ApiOperation(value = "用户管理", notes = "根据id查询")
    @GetMapping("/authority/getUserById/{id}")
    public ResultView getUserById(@PathVariable Long id) {
        UserInfo userPO = userInfoService.queryById(id);
        return ResultView.success(userPO);
    }

    @GetMapping("/authority/getUserList")
    @ApiOperation(value = "用户管理", notes = "查询list")
    public ResultView getUserList(@PathParam("orgCode") String orgCode, @PathParam("userRole") String userRole) {
        List<UserInfo> list = userInfoService.getUserList(orgCode, userRole);
        return ResultView.success(list);
    }


    @PostMapping("/authority/getUserPage")
    @ApiOperation(value = "用户管理", notes = "分页列表")
    public ResultView getUserPage(@RequestBody PageParam pageParam) {
        try {
            IPage<UserInfo> page = userInfoService.getUserPage(pageParam);
            return ResultView.success(page);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    @PostMapping("/authority/save")
    @ApiOperation(value = "用户管理", notes = "新增保存")
    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "新增用户信息")
    public ResultView saveUser(@RequestBody UserInfo userInfo) {
        userInfo.setId(YitIdHelper.nextId());
        userInfo.setBuildTime(DateUtils.getDate());
        userInfo.setUserPassword(MD5Util.getMD5(userInfo.getUserPassword()));
        boolean save = userInfoService.save(userInfo);
        if (save)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @PostMapping("/authority/update")
    @ApiOperation(value = "用户管理", notes = "根据id更新")
    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "修改用户信息")
    public ResultView updateUser(@RequestBody UserInfo userInfo) {
        userInfo.setBuildTime(DateUtils.getDate());
        userInfo.setUserPassword(MD5Util.getMD5(userInfo.getUserPassword()));
        boolean b = userInfoService.updateById(userInfo);
        if (b)
            return ResultView.success();
        else
            return ResultView.error();
    }

    @GetMapping("/authority/deleteById/{id}")
    @ApiOperation(value = "用户管理", notes = "根据id查询")
    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "删除用户信息")
    public ResultView getByIdForUserDelete(@PathVariable Long id) {
        if (userInfoService.removeById(id)) {
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @ApiOperation(value = "系统管理", notes = "登录")
    @PostMapping("/login")
    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "用户登录")
    public ResultView userLogin(@RequestParam String userName, @RequestParam String userPassword, HttpServletRequest request, @RequestParam String identifyCode) {
        try {
            if (!StringUtils.isEmpty(userName) && !StringUtils.isEmpty(userPassword)) {
                if (!StringUtils.isEmpty(identifyCode)) {
                    /*
                     * 从session中取出验证码
                     * 1. ServletContext:范围最大,应用程序级别的,整个应用程序都能访问
                     * 2. HttpSession–次之,会话级别的,在当前的浏览器中都能訪问[不论是在同一浏览器开多少窗体,都能够访问],可是换个浏览器就不行了,
                     *     就必须又一次创建session
                     * 3. HttpServletRequest–范围最小,请求级别,请求结束,变量的作用域也结束【也就是仅仅是一次访问,访问结束，这个也结束】
                     */
                    String sessionCode = (String) request.getServletContext().getAttribute("identifyCode");
                    if (identifyCode.equalsIgnoreCase(sessionCode)) {
                        // userId 为唯一值
                        UserInfo one = userInfoService.getOne(new QueryWrapper<UserInfo>().eq("user_name", userName));
                        if (Objects.nonNull(one)) {
                            String md5 = MD5Util.getMD5(userPassword);
                            if (one.getUserPassword().equals(md5)) {
                                request.getSession().setAttribute("userLogin", one);
                                // 密码使用完不返给前端(safest)
                                one.setUserPassword("");
                                // 用来存放payload信息
                                Map<String, String> map = new HashMap<>();
                                map.put("id", one.getId().toString());
                                map.put("username", one.getUserName());

                                // 生成token令牌
                                Map<String, Object> token = JWTUtil.generateToken(map, one);

                                return ResultView.success(token);
                            }
                        } else {
                            return ResultView.error(ResultEnum.CODE_5.getMsg());
                        }
                    } else {
                        return ResultView.error(ResultEnum.CODE_4.getMsg());
                    }
                } else {
                    return ResultView.error(ResultEnum.CODE_4_1.getMsg());
                }
            } else {
                return ResultView.error(ResultEnum.CODE_5_1.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultView.error(ResultEnum.CODE_2.getMsg());
    }

    /**
     * 给前端返回一个验证码图片
     */
    @Resource
    private IdentifyCodeUtils utils;

    @RequestMapping("/identifyImage")
    public void identifyImage(HttpServletResponse response, HttpServletRequest session) {
        //创建随机验证码
        String identifyCode = utils.getIdentifyCode();
        /*
         * session存入验证码
         * 1. ServletContext:范围最大,应用程序级别的,整个应用程序都能访问
         * 2. HttpSession–次之,会话级别的,在当前的浏览器中都能訪问[不论是在同一浏览器开多少窗体,都能够访问],可是换个浏览器就不行了,
         *     就必须又一次创建session
         * 3. HttpServletRequest–范围最小,请求级别,请求结束,变量的作用域也结束【也就是仅仅是一次访问,访问结束，这个也结束】
         */
        session.getServletContext().setAttribute("identifyCode", identifyCode);
        //根据验证码创建图片
        BufferedImage identifyImage = utils.getIdentifyImage(identifyCode);
        //回传给前端
        utils.responseIdentifyImg(identifyImage, response);
    }

}
