package com.unissoft.systemManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.productManage.mapper.ProductInfoMapper;
import com.unissoft.systemManage.mapper.HomePageMapper;
import com.unissoft.systemManage.model.AlarmStatistics;
import com.unissoft.systemManage.model.CPUMonitor;
import com.unissoft.systemManage.model.HomePage;
import com.unissoft.systemManage.service.HomePageService;
import com.unissoft.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author qingsong.han
 * @description:
 * @create 2022-06-29 11:30
 */
@Slf4j
@Service
public class HomePageServiceImpl implements HomePageService {
    @Resource
    private HomePageMapper homePageMapper;
    @Resource
    private ProductInfoMapper productInfoMapper;
    @Resource
    private RedisUtil redisUtil;

    @Override
    public HomePage getHomePageUp(String orgCode) {
        LocalDate now = LocalDate.now();
        String firstTime = String.format("%s %s", now.toString(), MyConstant.START_FLG);
        String endTime = String.format("%s %s", now.toString(), MyConstant.END_FLG);
        Map<String, String> map = new ConcurrentHashMap<>();
        // 组织机构编码
        map.put("orgCode", orgCode);
        // 当天开始时间
        map.put("firstTime", firstTime);
        // 当天最后一秒
        map.put("endTime", endTime);
        // 查询统计sql
        HomePage homePageUp = homePageMapper.getHomePageUp(map);
        /*
         * 设备在线数量,获取redis缓存实时数据
         * 1.查询所有orgCode下的产品编码
         * 2.根据产品编码统计在线设备
         */
        // 1.产品编码集合
        List<String> proSN = productInfoMapper.gotPorSNString(orgCode);
        // 2.在线设备数量
        int devOnlineSum = 0;
        for (String p : proSN) {
            if (redisUtil.hasKey(p)) {
                int l = redisUtil.hmget(p).size();
                devOnlineSum += l;
            }
        }
        homePageUp.setDevOnlineSum(devOnlineSum);
        // 设备总数量
        int devSum = homePageUp.getDevSum();
        // 设备总数量-在线数量=离线数量
        int devOfflineSum;
        // 判断未添加到设备列表的情况
        if (devSum <= 0 || devSum < devOnlineSum)
            devOfflineSum = 0;
        else
            devOfflineSum = devSum - devOnlineSum;
        homePageUp.setDevOfflineSum(devOfflineSum);
        if (devSum != 0) {
            // 在线率0
            homePageUp.setDevOnlineRate(Math.round((float) devOnlineSum / devSum * MyConstant.ONE_HUNDRED));
            // 离线率
            homePageUp.setDevOfflineRate(Math.round((float) devOfflineSum / devSum * MyConstant.ONE_HUNDRED));
        }
        return homePageUp;
    }

    /**
     * OperatingSystemMXBean:
     * {
     * "arch": "amd64",
     * "availableProcessors": 4,
     * "committedVirtualMemorySize": 1171865600,
     * "freePhysicalMemorySize": 6029860864,
     * "freeSwapSpaceSize": 2839302144,
     * "name": "Windows 10",
     * "objectName": {
     * "canonicalKeyPropertyListString": "type=OperatingSystem",
     * "domain": "java.lang",
     * "domainPattern": false,
     * "keyPropertyList": {
     * "type": "OperatingSystem"
     * },
     * "keyPropertyListString": "type=OperatingSystem",
     * "pattern": false,
     * "propertyListPattern": false,
     * "propertyPattern": false,
     * "propertyValuePattern": false    * 	},
     * "processCpuLoad": 0.0017797716123191295,
     * "processCpuTime": 27625000000,
     * "systemCpuLoad": 0.22316723432820373,
     * "systemLoadAverage": -1.0,
     * "totalPhysicalMemorySize": 17078149120,
     * "totalSwapSpaceSize": 19628285952,
     * "version": "10.0"
     * }
     *
     * @return cpu+new date
     */
    @Override
    public CPUMonitor getCPURate() {
        OperatingSystemMXBean operatingSystemMXBean = ManagementFactory.getOperatingSystemMXBean();
        String osJson = JSON.toJSONString(operatingSystemMXBean);
        log.info("java check system params logs for homepage: {}", osJson);
        JSONObject jsonObject = JSON.parseObject(osJson);
        CPUMonitor cpuMonitor = new CPUMonitor();
        cpuMonitor.setCpuUseRate(Math.round(jsonObject.getFloat("systemCpuLoad") * MyConstant.ONE_HUNDRED));
        cpuMonitor.setCpuLateUpdateTime(new Date());
        return cpuMonitor;
    }

    @Override
    public List<AlarmStatistics> getByTimeList(String orgCode, String optionTime) {
        Date dateNow = DateUtils.getDate();
        String firstTime = "";
        String endTime = "";
        if (!StringUtils.isEmpty(optionTime)) {
            if ("1".equals(optionTime)) { // 1小时
                // 获取当前时间 -1小时
                long time = dateNow.getTime() - MyConstant.ONE_HOUR;
                firstTime = dateJoining(time);
                endTime = DateUtils.dateToString(dateNow);
            } else if ("2".equals(optionTime)) { // 1天
                long time = dateNow.getTime() - MyConstant.ONE_DAY;
                firstTime = dateJoining(time);
                endTime = DateUtils.dateToString(dateNow);
            } else if ("3".equals(optionTime)) { // 7天
                long time = dateNow.getTime() - MyConstant.ONE_WEEK;
                firstTime = dateJoining(time);
                endTime = DateUtils.dateToString(dateNow);
            } else if ("4".equals(optionTime)) { // 30天
                long time = dateNow.getTime() - MyConstant.ONE_MONTH;
                firstTime = dateJoining(time);
                endTime = DateUtils.dateToString(dateNow);
            } else { // 日期
                firstTime = String.format("%s %s", optionTime, MyConstant.START_FLG);
                endTime = String.format("%s %s", optionTime, MyConstant.END_FLG);
            }

        }

        // 查询 条件
        Map<String, Object> map = new ConcurrentHashMap<>();
        map.put("orgCode", orgCode);
        map.put("firstTime", firstTime);
        map.put("endTime", endTime);
        // 获取时分秒
        /*String hms = DateUtils.getMdHMS(dateNow);
        System.out.println(hms);*/
        // TODO 结果集格式化(小时,天)
        return homePageMapper.getByTimeList(map);
    }

    private String dateJoining(long time) {
        // 转回date类型
        Date date = DateUtils.getDate(time);
        // 查询条件
        return DateUtils.dateToString(date);
    }

}
