package com.unissoft.systemManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.PageParam;
import com.unissoft.systemManage.mapper.LogOperationMapper;
import com.unissoft.systemManage.model.LogOperation;
import com.unissoft.systemManage.service.LogOperationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 操作日志表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-21
 */
@Service
public class LogOperationServiceImpl extends ServiceImpl<LogOperationMapper, LogOperation> implements LogOperationService {
    private LogOperationMapper operationMapper;

    @Autowired
    public void setLogOperationService(LogOperationMapper operationMapper) {
        this.operationMapper = operationMapper;
    }

    @Override
    public IPage<LogOperation> getLogPage(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        QueryWrapper<LogOperation> queryWrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            LogOperation operation = JSON.parseObject(condition, LogOperation.class);
            // 获取查询条件
            String orgCode = operation.getOrgCode();
            String userName = operation.getUserName();
            if (!StringUtils.isEmpty(orgCode)) {
                queryWrapper.eq("org_code", orgCode);
            }
            if (!StringUtils.isEmpty(userName)) {
                queryWrapper.like("user_name", userName);
            }
        }
        queryWrapper.orderByDesc("oper_time");
        // 分页对象
        return operationMapper.selectPage(new Page<>(pageIndex, pageSize), queryWrapper);
    }

}
