package com.unissoft.systemManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.unissoft.systemManage.mapper.OrgInfoMapper;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.OrgInfoChildren;
import com.unissoft.systemManage.service.OrgInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 组织机构和仓房表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@Service
public class OrgInfoServiceImpl extends ServiceImpl<OrgInfoMapper, OrgInfo> implements OrgInfoService {
    private OrgInfoMapper orgInfoMapper;

    @Autowired
    public void setOrgInfoService(OrgInfoMapper orgInfoMapper) {
        this.orgInfoMapper = orgInfoMapper;
    }

    @Override
    public List<OrgInfo> getList(OrgInfo orgInfo) {
        // 查询条件
//        QueryWrapper<OrgInfo> queryWrapper = new QueryWrapper<>(orgInfo);
        return orgInfoMapper.selectList(new QueryWrapper<>(orgInfo));
    }

    @Override
    public List<OrgInfoChildren> getTreeList(String orgName) {
        QueryWrapper<OrgInfo> queryWrapper = new QueryWrapper<>();
        List<OrgInfoChildren> orgInfoChildren = new ArrayList<>();
        String s;
        List<OrgInfo> orgInfoS;
        Long parentId = 0L;
        // 没有搜索条件时
        if (!StringUtils.isEmpty(orgName)) {
            queryWrapper.like("org_name", orgName);
            List<OrgInfo> orgInfoList1 = orgInfoMapper.selectList(queryWrapper);
            if (orgInfoList1.size() > 0) {
                // 二次查询list
                ArrayList<Long> ids = Lists.newArrayList(); // id缓存区
                for (OrgInfo orgInfo : orgInfoList1) {
                    ids.add(orgInfo.getId());
                }
                // 获取最顶级父id
                Optional<OrgInfo> min = orgInfoList1.stream().min(Comparator.comparingLong(OrgInfo::getParentId));
                if (min.isPresent()) {
                    parentId = min.get().getParentId();
                }
                // SELECT * FROM org_info WHERE id in(1,3) OR parent_id in(1,3)
                QueryWrapper<OrgInfo> query = new QueryWrapper<>();
                query.in("id", ids).or().in("parent_id", ids);
                orgInfoS = orgInfoMapper.selectList(query);
                s = JSON.toJSONString(orgInfoS);
                orgInfoChildren = JSON.parseArray(s, OrgInfoChildren.class);
            }
        } else { // 不使用orgName查询时(无查询条件时)
            orgInfoS = orgInfoMapper.selectList(queryWrapper);
            s = JSON.toJSONString(orgInfoS);
            orgInfoChildren = JSON.parseArray(s, OrgInfoChildren.class);
        }
        return buildTree(orgInfoChildren, parentId);
    }

    /**
     * 获取数据子集
     *
     * @param listParam 元数据
     * @param parentId  用于获取key 为最顶级 parentId
     * @return tree 结构 数据
     */
    private List<OrgInfoChildren> buildTree(List<OrgInfoChildren> listParam, Long parentId) {
        // 接收参数非空判断
        if (CollectionUtils.isEmpty(listParam)) {
            return Collections.emptyList();
        }
        // id 为key list 转map
        Map<Long, OrgInfoChildren> idMap = listParam.parallelStream().collect(Collectors.toMap(OrgInfoChildren::getId, p -> p));
        // 数据根据ParentId分组
        Map<Long, List<OrgInfoChildren>> pidGroupRolesMap = listParam.parallelStream().collect(Collectors.groupingBy(OrgInfoChildren::getParentId));
        // set children 值
        pidGroupRolesMap.forEach((k, v) -> {
            // 获取parent id key
            OrgInfoChildren parentRole = idMap.get(k);
            if (parentRole != null) {
                parentRole.setChildren(v);
            }
        });
        return pidGroupRolesMap.get(parentId);
    }

    @Override
    public Map<String, String> getOrgMap() {
        List<OrgInfo> list = orgInfoMapper.getOrgMap();
        return list.stream().collect(Collectors.toMap(OrgInfo::getOrgCode, OrgInfo::getOrgName));
    }

}
