package com.unissoft.visualisation.controller;

import com.unissoft.basic.model.AlarmParam;
import com.unissoft.common.ResultView;
import com.unissoft.deviceManage.model.DevicePoint;
import com.unissoft.visualisation.service.VisualisationService;
import com.unissoft.visualisation.service.impl.PointChild;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: 可视化
 * @create 2022-10-17 9:53
 */
@Slf4j
@Api(tags = "温湿度数据可视化")
@RestController
@RequestMapping("/vis")
public class VisualisationController {
    @Resource
    private VisualisationService visualisationService;

    @PostMapping("/getAlarmParamVis")
    @ApiOperation(value = "温湿度数据可视化报警参数", notes = "key->(温度:110001,湿度:110002)")
    public ResultView alarmParamVis(String orgCode, String labelCode) {
        try {
            Map<String, List<AlarmParam>> list = visualisationService.alarmParamVis(orgCode, labelCode);
            return ResultView.success(list);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    // 初始化点位数据
    @PostMapping("/initPointsDraw")
    @ApiOperation(value = "温湿度数据可视化点位初始化", notes = "获取pageSize条数据")
    public ResultView initPointsDraw(String orgCode, String labelCode, Integer pageSize) {
        try {
            // 返回有序点位数据->eqCode
            PointChild initPoints = visualisationService.initPointsDraw(orgCode, labelCode, pageSize);
            return ResultView.success(initPoints);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    // 12个月和一周历史趋势
    @PostMapping("/monthAndWeek")
    @ApiOperation(value = "温湿度数据可视化月和周趋势图", notes = "Integer round 月(2)或周(7)")
    public ResultView monthAndWeek(String orgCode, String labelCode, Integer round) {
        try {
            Map<String, Object> monthOrWeek = visualisationService.monthAndWeek(orgCode, labelCode, round);
            return ResultView.success(monthOrWeek);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

    // 预警信息
    @PostMapping("/alarmRecord")
    @ApiOperation(value = "温湿度数据可视化预警信息", notes = "获取pageSize条数据")
    public ResultView alarmRecord(String orgCode, String labelCode, Integer pageSize) {
        try {
            List<Map> alarmRecord = visualisationService.alarmRecord(orgCode, labelCode, pageSize);
            return ResultView.success(alarmRecord);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultView.error();
        }
    }

}
